#include "ezwin.h"
#include <cassert>
#include <cmath>
#define pi 3.14159265
#define powfact 0.02
#define ACCL 0.0001
#define rs 0.75 //Change radius of striker after modifications
#define rg 0.5
#define MS 1
#define MG 1
#define E 1

using namespace std;


SimpleWindow Carrom("Carrom", 28.0, 21.0,Position(0.0, 0.0));
float xm,ym,z,a,b,dist,vel;
int i=0,k=0;
float r=rs;
float x[20],y[20],vx[20],vy[20];

void Move();
void Collision();
void Carrom_Display();

void Move()
{
	i=0;
	r=rs;
	for(i=0;i<20;i++)
	{
		x[i]+=vx[i];
		if(x[i]<0)
			x[i]=0;
		if(x[i]>21-2*r)
			x[i]=21-2*r;

		y[i]+=vy[i];
		if(y[i]<0)
			y[i]=0;
		if(y[i]>21-2*r)
			y[i]=21-2*r;
		
		Carrom_Display();
		
		if(vx[i]==0 && vy[i]==0)
			continue;

		if((vx[i]*(vx[i]-ACCL*vx[i]/(double(pow(((double(pow(vx[i],2)))+(double(pow(vy[i],2)))),0.5)))))<=0)
		vx[i]=0;
		else
		vx[i]-=ACCL*vx[i]/(double(pow(((double(pow(vx[i],2)))+(double(pow(vy[i],2)))),0.5)));

		if((vy[i]*(vy[i]-ACCL*vy[i]/(double(pow(((double(pow(vx[i],2)))+(double(pow(vy[i],2)))),0.5)))))<=0)
		vy[i]=0;
		else
		vy[i]-=ACCL*vy[i]/(double(pow(((double(pow(vx[i],2)))+(double(pow(vy[i],2)))),0.5)));
		
		r=rg;
		
	}
}



void Collision()
{
	int i=0,j;
	float T,Pn,vi_n,vj_n,vi_t,vj_t;
	float d=rg+rs, M=MS, m=MG,r=rs,cx,cy;
	float D;
	do
	{
		for(j=i+1;j<20;j++)
		{
			D=double(pow(double(pow((x[i]-x[j]),2))+(double(pow((y[i]-y[j]),2))),0.5));
			if(D<d)
			{
					T=atan((y[i]-y[j])/(x[i]-x[j]));
					cx=(x[i]+x[j])/2;
					cy=(y[i]+y[j])/2;
					
					x[i]=cx+r*abs(cos(T))*(x[i]-cx)/abs(x[i]-cx);
					x[j]=cx+r*abs(cos(T))*(x[j]-cx)/abs(x[j]-cx);
					y[i]=cy+r*abs(sin(T))*(y[i]-cy)/abs(y[i]-cy);
					y[j]=cy+r*abs(sin(T))*(y[j]-cy)/abs(y[j]-cy);

					Carrom_Display();
					
					vi_n=vx[i]*cos(T)+vy[i]*sin(T);
					vi_t=vy[i]*cos(T)-vx[i]*sin(T);
					vj_n=vx[j]*cos(T)+vy[j]*sin(T);
					vj_t=vy[j]*cos(T)-vx[j]*sin(T);
					Pn= m*vj_n + M*vi_n;

					vi_n=(Pn-m*E*(vi_n-vj_n))/(M+m);
					vj_n=(Pn-M*E*(vj_n-vi_n))/(M+m);

					vx[j]=vj_n*cos(T)-vj_t*sin(T);
					vy[j]=vj_t*cos(T)+vj_n*sin(T);
					vx[i]=vi_n*cos(T)-vi_t*sin(T);
					vy[i]=vi_t*cos(T)+vi_n*sin(T);
			}			
		}
		
		if (y[i]==0 || (y[i]+2*r)==21)
			vy[i]=-E*vy[i];

		if (x[i]==0 || (x[i]+2*r)==21)
			vx[i]=-E*vx[i];

		
		i++;
		d=2*rg;
		r=rg;
		M=MG;
	}
	while(i<20);
}
/*
void Carrom_Display()
{
	Carrom.RenderRectangle(Position(0.0,0.0), Position(30.0,30.0), White, false);	//White Background
	Carrom.RenderRectangle(Position(3.0,1.5), Position(18.0,2.5), White, true);	//top

	for(i=0;i<20;i++)
	{
		if(i==0)
		{
		Carrom.RenderEllipse(Position(x[0],y[0]), Position(x[0]+2*rs,y[0]+2*rs), Cyan, true);
		Carrom.RenderEllipse(Position(x[0]+0.2,y[0]+0.2), Position(x[0]+2*rs-0.2,y[0]+2*rs-0.2),White, false);
		Carrom.RenderEllipse(Position(x[0]+0.4,y[0]+0.4), Position(x[0]+2*rs-0.4,y[0]+2*rs-0.4),Green, true);
		}
		else
		{
			Carrom.RenderEllipse( Position(x[i],y[i]),Position(x[i]+2*rg,y[i]+2*rg), Magenta, true);
		}
	}
}	
*/

void Carrom_Display()
{
	Carrom.RenderRectangle(Position(0.0,0.0), Position(30.0,30.0), White, false);	//White Background
//Holes
	Carrom.RenderEllipse(Position(-1.5,-1.5), Position(1.5,1.5), Black, true);
	Carrom.RenderEllipse(Position(-1.5,19.5), Position(1.5,22.5), Black, true);
	Carrom.RenderEllipse(Position(19.5,-1.5), Position(22.5,1.5), Black, true);
	Carrom.RenderEllipse(Position(19.5,19.5), Position(22.5,22.5), Black, true);

	Carrom.RenderRectangle(Position(21.0,0.0), Position(28.5,21.0), White, false);
	Carrom.RenderRectangle(Position(0.0,21.0), Position(25.0,25.0), White, false);
	Carrom.RenderLine(Position(0.0,21.0), Position(21.0,21.0), Black, 0.04);
	Carrom.RenderLine(Position(21.0,0.0), Position(21.0,21.0), Black, 0.04);
//White strips
	Carrom.RenderRectangle(Position(3.0,1.5), Position(18.0,2.5), White, true);	//top
	Carrom.RenderRectangle(Position(1.5,3.0), Position(2.5,18.0), White, true);	//Left
	Carrom.RenderRectangle(Position(18.5,3.0), Position(19.5,18.0), White, true);	//Right
	Carrom.RenderRectangle(Position(3.0,18.5), Position(18.0,19.5), White, true);	//Bottom
//Red Circles
	Carrom.RenderEllipse(Position(2.5,1.5), Position(3.5,2.5), White, true);	//top left
	Carrom.RenderEllipse(Position(2.6,1.6), Position(3.4,2.4), Red, true);	// -- 

	Carrom.RenderEllipse(Position(17.5,1.5), Position(18.5,2.5), White, true);	//top right
	Carrom.RenderEllipse(Position(17.6,1.6), Position(18.4,2.4), Red, true);	// --

	Carrom.RenderEllipse(Position(1.5,2.5), Position(2.5,3.5), White, true);	//Left top
	Carrom.RenderEllipse(Position(1.6,2.6), Position(2.4,3.4), Red, true);	// --

	Carrom.RenderEllipse(Position(1.5,17.5), Position(2.5,18.5), White, true);	//left bottom
	Carrom.RenderEllipse(Position(1.6,17.6), Position(2.4,18.4), Red, true);	// --

	Carrom.RenderEllipse(Position(18.5,2.5), Position(19.5,3.5), White, true);	//right top
	Carrom.RenderEllipse(Position(18.6,2.6), Position(19.4,3.4), Red, true);	// --

	Carrom.RenderEllipse(Position(18.5,17.5), Position(19.5,18.5), White, true);	//right bottom
	Carrom.RenderEllipse(Position(18.6,17.6), Position(19.4,18.4), Red, true);	// --

	Carrom.RenderEllipse(Position(2.5,18.5), Position(3.5,19.5), White, true);	//bottom left
	Carrom.RenderEllipse(Position(2.6,18.6), Position(3.4,19.4), Red, true);	// --

	Carrom.RenderEllipse(Position(17.5,18.5), Position(18.5,19.5), White, true);	//bottom right
	Carrom.RenderEllipse(Position(17.6,18.6), Position(18.4,19.4), Red, true);	// --
//CENTRE
	Carrom.RenderEllipse(Position(7.5,7.5), Position(13.5,13.5),White, true);
	Carrom.RenderEllipse(Position(7.6,7.6), Position(13.4,13.4),Magenta, true);		
	Carrom.RenderEllipse(Position(7.7,7.7), Position(13.3,13.3),White, true);
	Carrom.RenderEllipse(Position(8.5,8.5), Position(12.5,12.5),Red, false);
	Carrom.RenderEllipse(Position(8.7,8.7), Position(12.3,12.3),Yellow, true);
//STAR
	Carrom.RenderLine(Position(10.5,8.7), Position(8.9,11.3), Blue, 0.05);
	Carrom.RenderLine(Position(10.5,8.7), Position(12.1,11.3), Blue, 0.05);
	Carrom.RenderLine(Position(8.9,11.3), Position(12.1,11.3), Blue, 0.05);
	Carrom.RenderLine(Position(8.9,9.7), Position(12.1,9.7), Blue, 0.05);
	Carrom.RenderLine(Position(10.5,12.3), Position(8.9,9.7), Blue, 0.05);
	Carrom.RenderLine(Position(10.5,12.3), Position(12.1,9.7), Blue, 0.05);

	Carrom.RenderEllipse(Position(3.5,3.5), Position(5.5,5.5),White, true);
	Carrom.RenderEllipse(Position(15.5,3.5), Position(17.5,5.5),White, true);
	Carrom.RenderEllipse(Position(3.5,15.5), Position(5.5,17.5),White, true);
	Carrom.RenderEllipse(Position(15.5,15.5), Position(17.5,17.5),White, true);
//ARROWS
	Carrom.RenderLine(Position(1.5,1.5), Position(4.5,4.5), Black, 0.05);
	Carrom.RenderLine(Position(19.5,1.5), Position(16.5,4.5), Black, 0.05);
	Carrom.RenderLine(Position(1.5,19.5), Position(4.5,16.5), Black, 0.05);
	Carrom.RenderLine(Position(19.5,19.5), Position(16.5,16.5), Black, 0.05);

	for(i=0;i<20;i++)
	{
		if(i==0)
		{
			Carrom.RenderEllipse(Position(x[0],y[0]), Position(x[0]+2*rs,y[0]+2*rs), Cyan, true);
			Carrom.RenderEllipse(Position(x[0]+0.2,y[0]+0.2), Position(x[0]+2*rs-0.2,y[0]+2*rs-0.2),White, false);
			Carrom.RenderEllipse(Position(x[0]+0.4,y[0]+0.4), Position(x[0]+2*rs-0.4,y[0]+2*rs-0.4),Green, true);
		}
		else
		{
			Carrom.RenderEllipse( Position(x[i],y[i]),Position(x[i]+2*rg,y[i]+2*rg), Magenta, true);
		}
	}
}

//	Centre : (10.5,2.0)

int ApiMain() 
{	
	Carrom.Open();
	assert(Carrom.GetStatus() == WindowOpen);
	x[0]=10.5-rs;	y[0]=2.0-rs;

	x[1]=3;		y[1]=3;		x[11]=2;	y[11]=1;
	x[2]=5;		y[2]=8;		x[12]=3;	y[12]=3;
	x[3]=6;		y[3]=1;		x[13]=9;	y[13]=9;
	x[4]=12;	y[4]=14;	x[14]=18;	y[14]=6;
	x[5]=15;	y[5]=2;		x[15]=2;	y[15]=14;
	x[6]=10;	y[6]=6;		x[16]=6;	y[16]=15;
	x[7]=5;		y[7]=9;		x[17]=15;	y[17]=18;
	x[8]=9;		y[8]=7;		x[18]=14;	y[18]=17;
	x[9]=12;	y[9]=16;	x[19]=17;	y[19]=11;
	x[10]=19;	y[10]=13;	

	for(i=1;i<20;i++)
	{
		vx[i]=0;
		vy[i]=0;
	}
		
	Carrom_Display();

	int flag=1;

	cout<<"\nEnter position : ";	cin>>x[0];
	cout<<"\nEnter angle : ";	cin>>a;
	cout<<"\nEnter power : ";	cin>>vel;
	x[0]-=rs;
	b=a*pi/180;
	
	vx[0]=vel*cos(b);
	vy[0]=vel*sin(b);
	
	while(flag==1)
	{
		flag=0;
		Move();
		Collision();
		for(long int z=0;z<20000000;z++);
	
		for(int i=0;i<20;i++)
		{
			if((abs(vx[i])>0.0001)||abs(vy[i])>0.0001)
			{
				flag=1;
				break;
			}
		}
	}
	return 0;
}

